<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:xga="http://www.xgewerbeanzeige.de/spezifikation/2.0"
    xmlns:gc="http://docs.oasis-open.org/codelist/ns/genericode/1.0/"
    xmlns:datml="http://www.destatis.de/schema/datml-raw/2.0/de" xmlns:uuid="java:java.util.UUID"
    exclude-result-prefixes="xs datml uuid gc xsi" version="2.0">

    <xsl:output method="xml" indent="yes"/>
    <xsl:strip-space elements="*"/>

    <!-- 
         Dieses Skript transformiert gültige XGewerbeanzeige-Nachrichten der Fassung 1.3 zu XGA-Nachrichten der Version 2.0 (Proposal-Fassung).
         Es spiegelt den Stand am 23.07.2018 wider.
         
         Unterhalb dieses Kommentars kann eine Anzahl von Parametern verändert werden, mithilfe derer sich das Verhalten des 
         Skripts beeinflussen lässt:
         
         $verteilplattform-als-autor - Wenn 'true', wird für den Autor im Nachrichtenkopf das Präfix "egw" ausgegeben, sofern die
                                       Quellnachricht von einer Verteilplattform stammt. Wenn 'false', wird sie hingegen mit dem
                                       Präfix "MELDID" im Nachrichtenkopf ausgegeben, sofern in sämtlichen Sätzen identische Ge-
                                       meindeschlüssel verwendet werden, oder zurückgewiesen, falls unterschiedliche Gemeinde-
                                       schlüssel in den Sätzen angegeben sind.
                                       * Mögliche Werte: true, false
                                       
              $code-name-hinzufuegen - Wenn 'true', wird zum Autor (im Nachrichtenkopf und auf Satzebene) und Leser (Kopf)
                                       jeweils dasjenige Element behoerdenkennung/kennung/name erzeugt, das dem dort in jedem
                                       Fall angegebenen Element code entspricht. Eine Unterdrückung der (nicht-mandatorischen)
                                       Angabe des Namens kann wünschenswert sein bei der Stapelverarbeitung einer großen Menge
                                       von Nachrichten durch dieses Skript, da die Ermittlung des Namens (Codelisten-Literals)
                                       zum angegebenen Code die Konversion spürbar verlangsamt.
                                       * Mögliche Werte: true, false
                                       
               $textknoten-vergleich - Sucht auf Satzebene den Inhalt jedes Textknotens der Ursprunsgsnachricht in der Zielnachricht.
                                       Gibt es eine Abweichung, wird auf der Konsole das betroffene Element und sein Wert ausgegeben.
                                       Erwartete Ausnahmen (z.B. in 2.0 umgeformte Datumsangaben) werden herausgefiltert.
                                       * Mögliche Werte: true, false
                                       
            $im-fehlerfall-abbrechen - Wenn 'true', bricht das Skript die weitere Verarbeitung nach Ausgabe eines Fehlertextes ab;
                                       wenn 'false', wird nur der Fehlertext ausgegeben. Diese Einstellung dient dazu, bei einer
                                       Stapelverarbeitung von Nachrichten nach einer fehlerhaften, d.h. nicht transformierbaren
                                       Nachricht die verbleibenden dennoch zu verarbeiten (false) oder aber abzubrechen (true),
                                       um z.B. die Fehlersuche zu erleichtern. 
                                       * Mögliche Werte: true, false
                                       
                              $debug - Wenn 'true', werden übersprungene Elemente als Kommentare in die Zielstruktur eingefügt 
                                       sowie bei der Sortierung nicht gefundene Elemente auf der Konsole ausgegeben.
                                       * Mögliche Werte: true, false
                  
                            $verbose - Wenn 'true', wird zusätzlich jedes verarbeitete Element auf der Konsole ausgegeben (an-
                                       statt nur die nicht gefundenen). Hat nur einen Effekt, sofern $debug = true().
                                       * Mögliche Werte: true, false
                                       
    Aus Performancegründen und zur Gewährleistung eines vollen Durchlaufs sollten eingestellt werden: 
        "$code-name-hinzufuegen = 'false'" und "$im-fehlerfall-abbrechen = 'false'"
    -->

    <!-- *************************************** Parameter zur Steuerung des Skripts **************************************** -->
    <xsl:param name="verteilplattform-als-autor" as="xs:boolean">false</xsl:param>
    <xsl:param name="code-name-hinzufuegen" as="xs:boolean"
        >false<!-- mit true deutliche Verlangsamung bei Batch-Verarbeitung! --></xsl:param>
    <xsl:param name="textknoten-vergleich" as="xs:boolean">true</xsl:param>
    <xsl:param name="im-fehlerfall-abbrechen" as="xs:boolean">false</xsl:param>
    <xsl:param name="debug" as="xs:boolean">false</xsl:param>
    <xsl:param name="verbose" as="xs:boolean">false</xsl:param>
    <!-- ******************************************************************************************************************** -->

    <!-- Pfade zu den Typ-3-Codelisten, deren Codes zu Literalen aufgelöst werden, sofern $code-name-hinzufuegen = true() -->
    <xsl:variable name="codelisten-pfade" as="xs:string*"
        select="
            (
            'codelisten/AGS_Liste.xml',
            'codelisten/industrieundhandelskammern.xml',
            'codelisten/registergerichte.xml',
            'codelisten/unfallversicherungstraeger.xml',
            'codelisten/empfangsstellen-dvdv.xml',
            'codelisten/geschlecht.xml',
            'codelisten/handwerkskammern.xml'
            )
            "/>

    <xsl:variable name="codelisten" as="element(gc:CodeList)*"
        select="
            if ($code-name-hinzufuegen) then
                document($codelisten-pfade)/gc:CodeList
            else
                ()"/>

    <xsl:template match="/">
        <!-- Die Satzart der DatML/RAW-Quelle definiert das Wurzelelement in der Zielstruktur! -->
        <xsl:apply-templates select="//datml:hmm[@name = 'BerEmpfaenger']"/>
    </xsl:template>

    <!-- Ausgangspunkt: Generiert das Wurzelelement zur jeweiligen Satzart mitsamt zugehöriger Atribute und von dort aus,
         als Kindelemente, dessen Inhalte und speichert diese in einer Variable -->
    <xsl:template match="datml:hmm[@name = 'BerEmpfaenger']">

        <xsl:variable name="zielstruktur" as="node()*">
            <xsl:element name="xga:{xga:satzart(normalize-space(.))}">
                <!--<xsl:attribute name="xsi:schemaLocation">http://www.xgewerbeanzeige.de/spezifikation/2.0</xsl:attribute>-->
                <xsl:call-template name="xga:wurzelelement-attribute.produkt"/>
                <xsl:call-template name="xga:wurzelelement-attribute.produkthersteller"/>
                <xsl:call-template name="xga:wurzelelement-attribute.produktversion"/>
                <xsl:attribute name="standard">XGewerbeanzeige<!-- Konstante --></xsl:attribute>
                <xsl:attribute name="version">2.0<!-- Konstante --></xsl:attribute>
                <xsl:call-template name="xga:wurzelelement-attribute.test"/>
                <xsl:call-template name="xga:nachrichtenkopf"/>
                <xsl:call-template name="xga:berichtszeitraum"/>

                <!-- Erzeuge nacheinander die einzelnen Sätze der Nachricht. 
                     Der Parameter (die Satznummer) wird bei der Erzeugung des Autors auf Satzebene als Kontextinformation benötigt. -->
                <xsl:variable name="saetze-der-nachricht" as="node()*"
                    select="/datml:DatML-RAW-D/datml:nachricht/datml:datensegment//datml:satz"/>
                <xsl:for-each select="$saetze-der-nachricht">
                    <xsl:variable name="aktueller-satz" as="xs:integer" select="position()"/>
                    <xsl:apply-templates select="$saetze-der-nachricht[$aktueller-satz]"
                        mode="xga:satz">
                        <xsl:with-param name="satz-nummer" as="xs:integer" select="position()"/>
                    </xsl:apply-templates>
                </xsl:for-each>
            </xsl:element>
        </xsl:variable>

        <!-- Sofern Schalter gesetzt ist, zunächst noch Textknoten-Vergleich durchführen und ggf. Meldung zu Abweichungen ausgeben -->
        <xsl:if test="$textknoten-vergleich">
            <xsl:call-template name="textknotenvergleich">
                <xsl:with-param name="zielstruktur" select="$zielstruktur"/>
            </xsl:call-template>
        </xsl:if>

        <!-- Schließlich: Ausgabe der Zielstruktur -->
        <xsl:copy-of select="$zielstruktur"/>
    </xsl:template>

    <!-- Suche regelhaft für jeden Textknoten auf Satzebene der Quellstruktur sein Wert-Pendant in der Zielstruktur. 
         Erwartete Ausnahmen werden aus der Menge zu vergleichender Knoten herausgefiltert. -->
    <xsl:template name="textknotenvergleich">
        <xsl:param name="zielstruktur" as="node()*"/>
        <xsl:variable name="alle-zielknoten" as="xs:string*"
            select="$zielstruktur/xga:satz/*//text()/normalize-space(.)"/>

        <xsl:variable name="abweichungen-meldung">
            <xsl:for-each select="//datml:satz//datml:wert/text()">
                <xsl:variable name="gesuchter-text-in-quelle" as="xs:string"
                    select="normalize-space(.)"/>
                <xsl:variable name="beinhaltendes-elternelement" as="xs:string" select="../../@name"/>

                <xsl:choose>
                    <!-- Erwartete Ausnahmen: Die Inhalte der folgend aufgezählten Elemente werden nicht verglichen. Die Aufzählung muss 
                         von spezialisiert zu allgemein sortiert erfolgen, da nach der ersten zutreffenden xsl:when-Klausel die weiteren
                         nicht mehr ausgewertet werden! -->
                    <xsl:when
                        test="
                            $beinhaltendes-elternelement = ('Status', 'AusgestelltAm', 'AusgestelltVonName', 'AusgestelltVonPLZ', 'AusgestelltVonOrt', 'Auflagen')
                            and ancestor::datml:satz/datml:mm[@name = 'Satzart']/datml:wert = ('HWKID', 'IHKID')
                            (: gemäß XGA-448 entfällt Aufenthaltsgenehmigung mitsamt Kindelementen bei HWK und IHK :)"/>

                    <xsl:when
                        test="
                            $beinhaltendes-elternelement = 'EintragungNrSonstige'
                            and ancestor::datml:satz/datml:mm[@name = 'Satzart']/datml:wert = 'EAID'
                            (: I.6.2.9.1 Nachrichtenspezifische Einschränkungen der 2.0-Spezifikation :)"/>

                    <xsl:when
                        test="
                            $beinhaltendes-elternelement = (
                            'Satzart' (: entfällt :),
                            'GemeindeZusatz' (: entfällt :),
                            'MeldungArt' (: entfällt :),
                            'SatzErstellung' (: Inhalt wird im Ziel umgeformt :),
                            'AGSSchluesselVerzeichnis' (: entfällt :),
                            'DatumMeldung' (: Inhalt wird im Ziel umgeformt :),
                            'GueltigAb' (: Inhalt wird im Ziel umgeformt :),
                            'TelefonVorwahl' (: wird im Ziel zusammenfasst mit TelefonNummer :),
                            'TelefonNummer' (: wird im Ziel zusammenfasst mit TelefonVorwahl :),
                            'FaxVorwahl' (: wird im Ziel zusammenfasst mit FaxNummer :),
                            'FaxNummer' (: wird im Ziel zusammenfasst mit FaxVorwahl :),
                            'SGSSchluesselVerzeichnis' (: entfällt :),
                            'AusgestelltAm' (: Inhalt wird im Ziel umgeformt :)
                            )"/>

                    <!-- Auf Abweichung prüfen -->
                    <xsl:otherwise>
                        <xsl:if test="not($gesuchter-text-in-quelle = $alle-zielknoten)">
                            <xsl:value-of
                                select="('Zu ', local-name(../..), '[@name = ''', $beinhaltendes-elternelement, ''']/wert = ''', ., ''' wurde kein inhaltliches Pendant in der Zielstruktur gefunden!', '&#10;')"
                                separator=""/>
                        </xsl:if>

                        <!--
                            Der nachfolgende Häufigkeitsvergleich funktioniert, ist jedoch auskommentiert, weil er in Einzelfällen Hinweise produziert, in welchen
                            das Auftreten der Hinweise gar nicht vermeidbar ist und keinen Fehler darstellt: Z.B. entfällt aufgrund von XGA-448 in der 2.0-Variante in den Satzarten
                            IHK und HWK die in 1.3 vorhandene MMGR Aufenthaltsgenehmigung, wodurch in diesen Nachrichten dann die Textknoteninhalte der weggelassenen Kindelemente
                            fehlen, sodass sich zwingend eine Häufigkeitsdifferenz ergibt. Es müssten daher in einem dritten Filterschritt erneute Fallunterscheidungen
                            getroffen werden etc., was hier unverhältnismäßig scheint.
                                <!-\- Der Vergleich der Textknoten-Häufigkeiten ist für rein numerischen Knoteninhalt nicht sinnvoll, weil etliche Codelisten-Literale in 2.0
                                     existieren, die es in 1.3 nicht gab (z.B. "01"), oder beim Gemeindeschlüssel, der in 2.0 an mehr Stellen anzugeben ist als in 1.3.
                                     Daher werden nur alphanumerische Inhalte mit mindestens einem Buchstaben betrachtet -\->
                                <xsl:variable name="haeufigkeit-in-quelle" as="xs:integer"
                                    select="count(//datml:datensegment/descendant::text()[. = $gesuchter-text-in-quelle])"/>
                                <xsl:variable name="haeufigkeit-in-ziel" as="xs:integer"
                                    select="count($zielstruktur//xga:satz/descendant::text()/normalize-space(.)[. = $gesuchter-text-in-quelle])"/>
                                <xsl:variable name="suchtext-nicht-rein-numerisch" as="xs:boolean"
                                    select="string-length(translate($gesuchter-text-in-quelle, 'abcdefghijklmnopqrstuvwxyz0123456789', 'abcdefghijklmnopqrstuvwxyz')) != 0"/>
        
                                <xsl:if test="$haeufigkeit-in-quelle != $haeufigkeit-in-ziel and $suchtext-nicht-rein-numerisch">
                                    <xsl:value-of
                                        select="('Die Häufigkeit des Textknoteninhalts ''', ., ''' unterscheidet sich in Quelle (', $haeufigkeit-in-quelle, 'mal) und Ziel (', $haeufigkeit-in-ziel, 'mal)!', '&#10;')"
                                    />
                                </xsl:if>
                        -->
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </xsl:variable>

        <!-- Falls es Abweichungen zwischen den Textknoten von Quellstruktur (1.3-Nachricht) und Zielstruktur (2.0-Nachricht) gibt, werden sie auf der Konsole ausgegeben. -->
        <xsl:if test="not($abweichungen-meldung = '')">
            <!-- Überschrift mit betroffener Quelldatei ausgeben -->
            <xsl:message>
                <xsl:variable name="ueberschrift"
                    select="concat('Textuelle Abweichungen in Nachricht ', substring-after(document-uri(/), 'file:/'), ':')"/>
                <xsl:value-of select="($ueberschrift, '&#10;')"/>
                <xsl:value-of
                    select="
                        for $i in 1 to xs:integer(string-length($ueberschrift))
                        return
                            '*'"
                    separator=""/>
            </xsl:message>
            <!-- Abweichungen auflisten -->
            <xsl:message select="concat($abweichungen-meldung, '&#10;', ' ')"/>
        </xsl:if>
    </xsl:template>

    <!-- ***************************************
        Ab hier: Erzeugung des Nachrichtenkopfes
        **************************************** -->

    <xsl:template name="xga:wurzelelement-attribute.produkthersteller">
        <xsl:choose>
            <xsl:when
                test="/datml:DatML-RAW-D/datml:protokoll/datml:dokumentinstanz/datml:anwendung/datml:hersteller">
                <xsl:attribute name="produkthersteller">
                    <xsl:value-of
                        select="/datml:DatML-RAW-D/datml:protokoll/datml:dokumentinstanz/datml:anwendung/datml:hersteller"
                    />
                </xsl:attribute>
            </xsl:when>
            <xsl:otherwise>
                <xsl:attribute name="produkthersteller">KoSIT</xsl:attribute>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="xga:wurzelelement-attribute.produkt">
        <xsl:attribute name="produkt">
            <xsl:value-of
                select="/datml:DatML-RAW-D/datml:protokoll/datml:dokumentinstanz/datml:anwendung/datml:anwendungsname"
            />
        </xsl:attribute>
    </xsl:template>

    <xsl:template name="xga:wurzelelement-attribute.produktversion">
        <xsl:attribute name="produktversion">
            <xsl:value-of
                select="/datml:DatML-RAW-D/datml:protokoll/datml:dokumentinstanz/datml:anwendung/datml:version"
            />
        </xsl:attribute>
    </xsl:template>

    <xsl:template name="xga:wurzelelement-attribute.test">
        <xsl:if test="/datml:DatML-RAW-D/datml:optionen/datml:test/@kennung">
            <xsl:attribute name="test">
                <xsl:value-of select="/datml:DatML-RAW-D/datml:optionen/datml:test/@kennung"/>
            </xsl:attribute>
        </xsl:if>
    </xsl:template>

    <xsl:template name="xga:leser">
        <xga:leser>
            <xsl:call-template name="xga:behoerdenkennung-leser"/>
            <xga:behoerdenname>
                <xsl:value-of
                    select="(/datml:DatML-RAW-D/datml:empfaenger/datml:identifikation/datml:identitaet/datml:organisation/datml:name, '(KEINE LESENDE BEHÖRDE ANGEGEBEN IN QUELLNACHRICHT)')[1]"
                />
            </xga:behoerdenname>
        </xga:leser>
    </xsl:template>

    <xsl:template name="xga:behoerdenkennung-leser">
        <xga:behoerdenkennung>
            <xsl:choose>
                <!-- Leser-Präfix: xga -->
                <xsl:when
                    test="/datml:DatML-RAW-D/datml:empfaenger/datml:externeIdentifikation/@klasse = 'xga'">
                    <xga:praefix listVersionID="7">
                        <code>xga</code>
                    </xga:praefix>
                    <xsl:variable name="urn" as="xs:string"
                        select="'urn:xoev-de:xgewerbeanzeige:codeliste:empfangsstellen.dvdv'"/>
                    <xga:kennung listURI="{$urn}" listVersionID="{xga:listVersionID($urn)}">
                        <code>
                            <xsl:value-of
                                select="/datml:DatML-RAW-D/datml:empfaenger/datml:externeIdentifikation"
                            />
                        </code>
                        <xsl:if test="$code-name-hinzufuegen">
                            <name>
                                <xsl:value-of
                                    select="xga:codelisten-wert($urn, /datml:DatML-RAW-D/datml:empfaenger/datml:externeIdentifikation, 'Bezeichnung')"
                                />
                            </name>
                        </xsl:if>
                    </xga:kennung>
                </xsl:when>

                <!-- Leser-Präfix: MELDID -->
                <xsl:when
                    test="/datml:DatML-RAW-D/datml:empfaenger/datml:externeIdentifikation/@klasse = 'MELDID'">
                    <!-- Es handelt sich um eine Gewerbehörde. Die nachfolgende Kennung ist ein AGS, für den diese Gewerbebehörde zuständig ist -->
                    <xga:praefix listVersionID="7">
                        <code>MELDID</code>
                    </xga:praefix>
                    <xsl:variable name="urn" as="xs:string"
                        select="'urn:de:bund:destatis:bevoelkerungsstatistik:schluessel:ags'"/>
                    <xga:kennung listURI="{$urn}"
                        listVersionID="{/datml:DatML-RAW-D/datml:nachricht/datml:datensegment/datml:satz[1]/datml:mm[@name = 'AGSSchluesselVerzeichnis']/datml:wert/tokenize(., '_')[2]}">
                        <code>
                            <xsl:value-of
                                select="/datml:DatML-RAW-D/datml:empfaenger/datml:externeIdentifikation"
                            />
                        </code>
                        <xsl:if test="$code-name-hinzufuegen">
                            <name>
                                <xsl:value-of
                                    select="xga:codelisten-wert($urn, /datml:DatML-RAW-D/datml:empfaenger/datml:externeIdentifikation, 'Bezeichnung')"
                                />
                            </name>
                        </xsl:if>
                    </xga:kennung>
                </xsl:when>

                <!-- Leser-Präfix: egw -->
                <xsl:when
                    test="/datml:DatML-RAW-D/datml:empfaenger/datml:externeIdentifikation/@klasse = 'egw'">
                    <!-- Es handelt sich um ein zentrales Landessystem, welches Gewerbebehörden des Landes elektronisch gegenüber anderen Stellen vertritt. 
                         Die nachfolgende Kennung ist ein Element der Codeliste der Länderschlüssel. -->
                    <xga:praefix listVersionID="7">
                        <code>egw</code>
                    </xga:praefix>
                    <xsl:variable name="urn" as="xs:string"
                        select="'urn:xoev-de:xgewerbeanzeige:codeliste:landessysteme.dvdv'"/>
                    <xga:kennung listURI="{$urn}" listVersionID="1">
                        <code>
                            <xsl:value-of
                                select="/datml:DatML-RAW-D/datml:empfaenger/datml:externeIdentifikation"
                            />
                        </code>
                        <xsl:if test="$code-name-hinzufuegen">
                            <name>DUMMY-WERT</name>
                        </xsl:if>
                    </xga:kennung>
                </xsl:when>

                <!-- Leser-Präfix: eGewSpDB (Dataport) -->
                <xsl:when
                    test="/datml:DatML-RAW-D/datml:empfaenger/datml:externeIdentifikation/@klasse = 'eGewSpDB'">
                    <!-- Hier wird Dataport als Verteilplattform adressiert. -->
                    <xga:praefix listVersionID="7">
                        <code>egw</code>
                    </xga:praefix>
                    <xsl:variable name="urn" as="xs:string"
                        select="'urn:xoev-de:xgewerbeanzeige:codeliste:landessysteme.dvdv'"/>
                    <xga:kennung listURI="{$urn}" listVersionID="{xga:listVersionID($urn)}">

                        <!-- Da "99" nicht mehr zulässig ist, werden für den Code die ersten beiden Stellen des AGS des ersten Satzes herangezogen  -->
                        <xsl:variable name="umgeschluesselter-ags" as="xs:string"
                            select="/datml:DatML-RAW-D/datml:nachricht/datml:datensegment/datml:satz[1]/datml:mm[@name = 'GemeindeSchluessel']/datml:wert/substring(., 1, 2)"/>

                        <code>
                            <xsl:value-of select="$umgeschluesselter-ags"/>
                        </code>
                        <xsl:if test="$code-name-hinzufuegen">
                            <name>DUMMY-WERT</name>
                        </xsl:if>
                    </xga:kennung>
                </xsl:when>
            </xsl:choose>
        </xga:behoerdenkennung>
    </xsl:template>

    <xsl:template name="xga:behoerdenkennung-autor">
        <!-- Die Erzeugung der Behördenkennung des Autors auf Ebene des Nachrichtenkopfes bedarf einer Fallunterscheidung:
            
             (1a) Absender ist MELDID und alle Sätze haben den gleichen AGS (nicht zwingend den aus dem Kopf)
                  => Autor-Behördenkennung im Kopf aus dem AGS der Sätze bilden
             (1b) Absender ist MELDID, aber es haben nicht alle Sätze (untereinander) den gleichen AGS
                  => Abbruch mit Fehlermeldung 
             (2a) Absender ist Verteilplattform, alle Sätze in der Nachricht haben den gleichen AGS, Schalter "verteilplattform-als-autor" nicht gesetzt
                  => aus AGS kann der Autor im Kopf (MELDID) erzeugt werden
             (2b) Absender ist Verteilplattform, die Sätze in der Nachricht haben unterschiedliche AGS, Schalter nicht gesetzt 
                  => Abbruch mit Fehlermeldung
             (2c) Absender ist Verteilplattform (Variante Gewan) und Schalter gesetzt 
                  => Autor ist Verteilplattform (egw)              
             (2d) Absender ist Verteilplattform und Schalter gesetzt 
                  => Autor ist Verteilplattform (egw)              
        -->
        <xga:behoerdenkennung>
            <!-- Ein ggf. als MELDID übergebener 12-stelliger Regionalschlüssel (Dataport) wird in einen 8-stelligen AGS umgewandelt -->
            <xsl:variable name="kennung-original" as="xs:string"
                select="/datml:DatML-RAW-D/datml:absender/datml:kennung"/>
            <xsl:variable name="kennung" as="xs:string"
                select="
                    if (string-length($kennung-original) = 12) then
                        xga:meldid12-zu-ags8($kennung-original)
                    else
                        $kennung-original"/>

            <xsl:choose>
                <!-- (1a) Absender ist MELDID und alle Sätze haben den gleichen AGS (nicht zwingend den aus dem Kopf):
                          => Autor-Behördenkennung im Kopf aus dem AGS der Sätze bilden -->
                <xsl:when
                    test="/datml:DatML-RAW-D/datml:absender/datml:kennung[@klasse = 'MELDID' and count(distinct-values(/datml:DatML-RAW-D//datml:mm[@name = 'GemeindeSchluessel']/datml:wert)) = 1]">

                    <xga:praefix listVersionID="7">
                        <code>MELDID</code>
                    </xga:praefix>

                    <xsl:variable name="urn" as="xs:string"
                        select="'urn:de:bund:destatis:bevoelkerungsstatistik:schluessel:ags'"/>
                    <xsl:variable name="ags-schluessel" as="xs:string?"
                        select="(/datml:DatML-RAW-D//datml:mm[@name = 'GemeindeSchluessel']/datml:wert)[1]"/>

                    <xga:kennung listURI="{$urn}" listVersionID="{xga:listVersionID($urn)}">
                        <code>
                            <xsl:value-of select="$ags-schluessel"/>
                        </code>
                        <xsl:if test="$ags-schluessel and $code-name-hinzufuegen">
                            <name>
                                <xsl:value-of
                                    select="xga:codelisten-wert($urn, $ags-schluessel, 'Bezeichnung')"
                                />
                            </name>
                        </xsl:if>
                    </xga:kennung>
                </xsl:when>

                <!-- (1b) Absender ist MELDID, aber es haben nicht alle Sätze (untereinander) den gleichen AGS:
                          => Abbruch mit Fehlermeldung -->
                <xsl:when test="/datml:DatML-RAW-D/datml:absender/datml:kennung[@klasse = 'MELDID']">
                    <xsl:message terminate="{if ($im-fehlerfall-abbrechen) then 'yes' else 'no'}"
                        select="'Eine Nachricht der Behördenkennung MELDID kann nicht nach 2.0 übersetzt werden, wenn in den Sätzen verschiedene AGS verwendet werden: Die Nachricht müsste vorher geeignet aufgeteilt werden.'"
                    />
                </xsl:when>

                <!-- (2a) Absender ist Verteilplattform, identischer AGS in allen Sätzen, Schalter "verteilplattform-als-autor" nicht gesetzt:
                          => aus AGS kann der Autor im Kopf (MELDID) erzeugt werden -->
                <xsl:when
                    test="/datml:DatML-RAW-D/datml:absender/datml:kennung[@klasse = 'egw' and count(distinct-values(/datml:DatML-RAW-D//datml:mm[@name = 'GemeindeSchluessel'])) = 1 and $verteilplattform-als-autor = false()]">
                    <xga:praefix listVersionID="7">
                        <code>MELDID</code>
                    </xga:praefix>

                    <xsl:variable name="urn" as="xs:string"
                        select="'urn:de:bund:destatis:bevoelkerungsstatistik:schluessel:ags'"/>
                    <xsl:variable name="ags-schluessel" as="xs:string"
                        select="(/datml:DatML-RAW-D//datml:mm[@name = 'GemeindeSchluessel']/datml:wert)[1]"/>

                    <xga:kennung listURI="{$urn}" listVersionID="{xga:listVersionID($urn)}">
                        <code>
                            <xsl:value-of select="$ags-schluessel"/>
                        </code>
                        <xsl:if test="$code-name-hinzufuegen">
                            <name>
                                <xsl:value-of
                                    select="xga:codelisten-wert($urn, $ags-schluessel, 'Bezeichnung')"
                                />
                            </name>
                        </xsl:if>
                    </xga:kennung>
                </xsl:when>

                <!-- (2b) Absender ist Verteilplattform, unterschiedliche AGS in den Sätzen, Schalter "verteilplattform-als-autor" nicht gesetzt:
                          => Abbruch mit Fehlermeldung -->
                <xsl:when
                    test="/datml:DatML-RAW-D/datml:absender/datml:kennung[@klasse = 'egw' and count(distinct-values(/datml:DatML-RAW-D//datml:mm[@name = 'GemeindeSchluessel'])) > 1 and $verteilplattform-als-autor = false()]">
                    <xsl:message terminate="{if ($im-fehlerfall-abbrechen) then 'yes' else 'no'}"
                        select="'Eine Nachricht der Behördenkennung egw kann nicht nach 2.0 übersetzt werden, wenn in den Sätzen verschiedene AGS verwendet werden: Die Nachricht müsste vorher geeignet aufgeteilt werden.'"
                    />
                </xsl:when>

                <!-- (2c) Absender ist Verteilplattform (Gewan), Schalter "verteilplattform-als-autor" gesetzt:
                          => Autor ist Verteilplattform (egw) -->
                <xsl:when
                    test="/datml:DatML-RAW-D/datml:absender/datml:kennung[@klasse = 'MELDID' and string-length(.) = 2 and $verteilplattform-als-autor = true()]">
                    <xga:praefix listVersionID="7">
                        <code>egw</code>
                    </xga:praefix>
                    <xsl:variable name="urn" as="xs:string"
                        select="'urn:de:bund:destatis:bevoelkerungsstatistik:schluessel:bundesland'"/>
                    <xga:kennung listURI="{$urn}" listVersionID="{xga:listVersionID($urn)}">
                        <!-- Es handelt sich um ein zentrales Landessystem, welches Gewerbebehörden des Landes elektronisch gegenüber anderen Stellen vertritt. 
                         Die nachfolgende Kennung ist ein Element der Codeliste der Länderschlüssel -->
                        <code>
                            <xsl:value-of select="substring($kennung, 1, 2)"/>
                        </code>
                        <xsl:if test="$code-name-hinzufuegen">
                            <name>
                                <xsl:value-of
                                    select="xga:codelisten-wert($urn, $kennung, 'Bezeichnung')"/>
                            </name>
                        </xsl:if>
                    </xga:kennung>
                </xsl:when>

                <!-- (2d) Absender ist Verteilplattform, Schalter "verteilplattform-als-autor" gesetzt:
                          => Autor ist Verteilplattform (egw) -->
                <xsl:when
                    test="/datml:DatML-RAW-D/datml:absender/datml:kennung[@klasse = 'egw' and $verteilplattform-als-autor = true()]">
                    <xga:praefix listVersionID="7">
                        <code>egw</code>
                    </xga:praefix>
                    <xsl:variable name="urn" as="xs:string"
                        select="'urn:de:bund:destatis:bevoelkerungsstatistik:schluessel:bundesland'"/>
                    <xga:kennung listURI="{$urn}" listVersionID="{xga:listVersionID($urn)}">
                        <!-- Es handelt sich um ein zentrales Landessystem, welches Gewerbebehörden des Landes elektronisch gegenüber anderen Stellen vertritt. 
                         Die nachfolgende Kennung ist ein Element der Codeliste der Länderschlüssel -->
                        <code>
                            <xsl:value-of select="$kennung"/>
                        </code>
                        <xsl:if test="$code-name-hinzufuegen">
                            <name>
                                <xsl:value-of
                                    select="xga:codelisten-wert($urn, $kennung, 'Bezeichnung')"/>
                            </name>
                        </xsl:if>
                    </xga:kennung>
                </xsl:when>


                <!-- In allen anderen Fällen: Abbruch -->
                <xsl:otherwise>
                    <xsl:message terminate="{if ($im-fehlerfall-abbrechen) then 'yes' else 'no'}"
                        select="'Für diese Kennung bzw. diese externe Identifikation kann keine Behördenkennung in 2.0 erzeugt werden.'"
                    />
                </xsl:otherwise>
            </xsl:choose>
        </xga:behoerdenkennung>
    </xsl:template>

    <!-- In 1.3. gibt es auf Kopfebene als die Erreichbarkeit betreffende Absenderangaben lediglich Telefonnummer und E-Mail-Adresse -->
    <xsl:template name="xga:erreichbarkeit-autor">
        <xsl:if test="/datml:DatML-RAW-D/datml:absender/datml:kontakt/datml:email">
            <xga:erreichbarkeit>
                <xga:kanal>
                    <code>01<!-- E-Mail-Adresse --></code>
                </xga:kanal>
                <xga:kennung>
                    <xsl:value-of
                        select="/datml:DatML-RAW-D/datml:absender/datml:kontakt/datml:email"/>
                </xga:kennung>
            </xga:erreichbarkeit>
        </xsl:if>
        <xsl:if test="/datml:DatML-RAW-D/datml:absender/datml:kontakt/datml:telefon">
            <xga:erreichbarkeit>
                <xga:kanal>
                    <code>02<!-- Telefonnummer --></code>
                </xga:kanal>
                <xga:kennung>
                    <xsl:value-of
                        select="/datml:DatML-RAW-D/datml:absender/datml:kontakt/datml:telefon"/>
                </xga:kennung>
            </xga:erreichbarkeit>
        </xsl:if>
    </xsl:template>

    <xsl:template name="xga:anschrift-autor">
        <xga:anschrift>
            <xsl:for-each
                select="/datml:DatML-RAW-D/datml:absender/datml:identifikation/datml:adresse/datml:strasse">
                <xga:strasse>
                    <xsl:value-of select="."/>
                </xga:strasse>
            </xsl:for-each>

            <xsl:for-each
                select="/datml:DatML-RAW-D/datml:absender/datml:identifikation/datml:adresse/datml:hausnummer">
                <xga:hausNr>
                    <xsl:value-of select="."/>
                </xga:hausNr>
            </xsl:for-each>

            <xsl:for-each
                select="/datml:DatML-RAW-D/datml:absender/datml:identifikation/datml:adresse/datml:ort">
                <xga:ort>
                    <xsl:value-of select="."/>
                </xga:ort>
            </xsl:for-each>

            <xsl:for-each
                select="/datml:DatML-RAW-D/datml:absender/datml:identifikation/datml:adresse/datml:postleitzahl">
                <xga:plz>
                    <xsl:value-of select="."/>
                </xga:plz>
            </xsl:for-each>
        </xga:anschrift>
    </xsl:template>

    <xsl:template name="xga:behoerdenname-autor">
        <xga:behoerdenname>
            <xsl:value-of
                select="/datml:DatML-RAW-D/datml:absender/datml:identifikation/datml:identitaet/datml:organisation/datml:name"
            />
        </xga:behoerdenname>
    </xsl:template>

    <xsl:template name="xga:autor">
        <xga:autor>
            <xsl:call-template name="xga:behoerdenkennung-autor"/>
            <xsl:call-template name="xga:erreichbarkeit-autor"/>
            <xsl:call-template name="xga:anschrift-autor"/>
            <xsl:call-template name="xga:behoerdenname-autor"/>
        </xga:autor>
    </xsl:template>

    <!-- Wandelt das DatML/RAW-Hilfsmerkmal 'BerEmpfaenger' (= Satzart) in das zugehörige Wurzelelement in XGA 2.0 -->
    <xsl:function name="xga:satzart" as="xs:string">
        <xsl:param name="DatML-BerEmpfaenger" as="xs:string"/>
        <xsl:choose>
            <xsl:when test="$DatML-BerEmpfaenger = 'STAID'">sta.gewerbemeldung.0100</xsl:when>
            <xsl:when test="$DatML-BerEmpfaenger = 'IHKID'">ihk.gewerbemeldung.0200</xsl:when>
            <xsl:when test="$DatML-BerEmpfaenger = 'HWKID'">hwk.gewerbemeldung.0210</xsl:when>
            <xsl:when test="$DatML-BerEmpfaenger = 'EAID'">ea.gewerbemeldung.0220</xsl:when>
            <xsl:when test="$DatML-BerEmpfaenger = 'DGUID'">dgu.gewerbemeldung.0230</xsl:when>
            <xsl:when test="$DatML-BerEmpfaenger = 'RGID'">rg.gewerbemeldung.0240</xsl:when>
            <xsl:when test="$DatML-BerEmpfaenger = 'FAID'">fa.gewerbemeldung.0250</xsl:when>
            <xsl:when test="$DatML-BerEmpfaenger = 'ZVID'">zv.gewerbemeldung.0260</xsl:when>
            <xsl:when test="$DatML-BerEmpfaenger = 'ISID'">is.gewerbemeldung.0270</xsl:when>
            <xsl:when test="$DatML-BerEmpfaenger = 'ASID'">as.gewerbemeldung.0280</xsl:when>
            <xsl:when test="$DatML-BerEmpfaenger = 'LUEID'">lue.gewerbemeldung.0290</xsl:when>
            <xsl:when test="$DatML-BerEmpfaenger = 'ALGID'">alg.gewerbemeldung.0300</xsl:when>
            <xsl:otherwise>
                <xsl:message>
                    <xsl:value-of
                        select="concat('Nicht implementierte Satzart ', $DatML-BerEmpfaenger, '. Kein geeignetes Wurzelelement vorhanden.')"
                    />
                </xsl:message>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <!-- Erzeuge Nachrichtenkopf. -->
    <xsl:template name="xga:nachrichtenkopf">
        <xga:nachrichtenkopf>
            <xga:identifikation.nachricht>
                <xsl:call-template name="xga:nachrichtenUUID"/>
                <xsl:call-template name="xga:nachrichtentyp"/>
                <xsl:call-template name="xga:erstellungszeitpunkt"/>
            </xga:identifikation.nachricht>
            <xsl:call-template name="xga:leser"/>
            <xsl:call-template name="xga:autor"/>
        </xga:nachrichtenkopf>
    </xsl:template>

    <xsl:template name="xga:nachrichtenUUID">
        <!-- Zu Vergleichszwecken wird momentan stets dieselbe "UUID" verwendet. -->
        <xga:nachrichtenUUID>00000000-0000-0000-0000-000000000001</xga:nachrichtenUUID>
    </xsl:template>

    <xsl:template name="xga:nachrichtentyp">
        <xga:nachrichtentyp>
            <code>
                <!-- Der Nachrichtentyp-Code entspricht dem Nummernkreis, d.h. dem Ziffernblock hinter dem zweiten Punkt des Nachrichtentyps -->
                <xsl:value-of
                    select="tokenize(xga:satzart(normalize-space(/datml:DatML-RAW-D/datml:nachricht/datml:hmm[@name = 'BerEmpfaenger'])), '\.')[3]"
                />
            </code>
        </xga:nachrichtentyp>
    </xsl:template>

    <xsl:template name="xga:erstellungszeitpunkt">
        <xga:erstellungszeitpunkt>
            <xsl:value-of
                select="
                    xga:xsdatetime(
                    /datml:DatML-RAW-D/datml:protokoll/datml:dokumentinstanz/datml:datum,
                    /datml:DatML-RAW-D/datml:protokoll/datml:dokumentinstanz/datml:uhrzeit
                    )"
            />
        </xga:erstellungszeitpunkt>
    </xsl:template>

    <!-- Wandelt die parametrisierten Datums- und Uhrzeitstrings in das Zielformat um: '27.10.2017', '19:15:24' => 2017-10-27T19:15:24.0 -->
    <xsl:function name="xga:xsdatetime" as="xs:string">
        <xsl:param name="datum" as="xs:string"/>
        <xsl:param name="uhrzeit" as="xs:string"/>
        <xsl:variable name="datumbestandteile" as="xs:string*" select="tokenize($datum, '\.')"/>
        <xsl:value-of
            select="concat($datumbestandteile[3], '-', $datumbestandteile[2], '-', $datumbestandteile[1], 'T', $uhrzeit, '.0')"
        />
    </xsl:function>

    <!-- Der Berichtszeitraum wird in 2.0 nur noch für die Satzart STAID übermittelt (XGA-518) -->
    <xsl:template name="xga:berichtszeitraum">
        <xsl:if test="../datml:hmm[@name = 'BerEmpfaenger']/datml:wert = 'STAID'">
            <xga:berichtszeitraum>
                <xsl:value-of
                    select="
                        concat(
                        /datml:DatML-RAW-D/datml:nachricht/datml:berichtszeitraum/datml:jahr, '-',
                        /datml:DatML-RAW-D/datml:nachricht/datml:berichtszeitraum/datml:monat
                        )"
                />
            </xga:berichtszeitraum>
        </xsl:if>
    </xsl:template>

    <!-- Erzeugt die Sätze -->
    <xsl:template match="datml:satz" mode="xga:satz">
        <xsl:param name="satz-nummer" as="xs:integer" required="yes"/>
        <xga:satz>
            <!-- Referenznachrichten enthalten an dieser Stelle einen Kommentar: Diesen der Übersichtlichkeit halber nach 2.0 übernehmen. -->
            <xsl:if test="./comment()">
                <xsl:copy-of select="./comment()"/>
            </xsl:if>

            <!-- <anmeldung>, <ummeldung> oder <abbmeldung> erzeugen, und darin die Satzinhalte -->
            <xsl:element name="xga:{xga:meldungsart(datml:mm[@name='MeldungArt'])}">
                <xsl:call-template name="satz-inhalt">
                    <xsl:with-param name="satz-nummer" select="$satz-nummer"/>
                </xsl:call-template>
            </xsl:element>
        </xga:satz>
    </xsl:template>

    <!-- Wandelt das Meldungsart-Literal in DatML/RAW in dan korrespondierenden XGA 2.0-Typ um -->
    <xsl:function name="xga:meldungsart" as="xs:string">
        <xsl:param name="meldungnr" as="xs:integer"/>
        <xsl:choose>
            <xsl:when test="$meldungnr = 1">anmeldung</xsl:when>
            <xsl:when test="$meldungnr = 2">ummeldung</xsl:when>
            <xsl:when test="$meldungnr = 3">abmeldung</xsl:when>
            <xsl:otherwise>
                <xsl:message>
                    <xsl:value-of
                        select="concat('Unbekanntes Meldungsart-Literal: ', $meldungnr, '. Kein Meldungstyp zuordenbar.')"
                    />
                </xsl:message>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <!-- **************************************************************************
        Ab hier: Erzeugung der Satz-Inhalte in korrekter Reihenfolge gemäß XGA 2.0
        ************************************************************************** -->
    <xsl:template name="satz-inhalt">
        <xsl:param name="satz-nummer" as="xs:integer" required="yes"/>

        <xsl:variable name="satzart"
            select="/datml:DatML-RAW-D/datml:nachricht/datml:datensegment/datml:satz[$satz-nummer]/datml:mm[@name = 'Satzart']/datml:wert"/>
        <xsl:apply-templates select="datml:mmgr | datml:mm">
            <xsl:with-param name="satz-nummer" as="xs:integer" select="$satz-nummer"/>
            <xsl:sort select="xga:lokale-position-fuer-element($satzart, @name)"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="datml:mm">
        <xsl:element name="xga:{lower-case(substring(@name,1,1))}{substring(@name, 2)}">
            <xsl:value-of select="datml:wert"/>
        </xsl:element>
    </xsl:template>

    <!-- Erzeuge datml:Foo (1.3) als xga:foo (2.0) in der geforderten Reihenfolge. Etwaige Sonderfälle (abweichende Struktur) werden in eigenen Templates abgebildet -->
    <xsl:template match="datml:mmgr">
        <xsl:variable name="current-mmgr" select="."/>
        <xsl:element name="xga:{lower-case(substring(@name,1,1))}{substring(@name, 2)}">
            <xsl:apply-templates select="*">
                <xsl:sort select="xga:lokale-position-fuer-element($current-mmgr/@name, @name)"/>
            </xsl:apply-templates>
        </xsl:element>
    </xsl:template>

    <!-- Hilfsfunktion im Zusammenhang der Erzeugung der Elemente in der von 2.0 geforderten Reihenfolge -->
    <xsl:function name="xga:lokale-position-fuer-element" as="xs:integer?">
        <xsl:param name="direktes-elternelement" as="xs:string"/>
        <!-- satz oder mmgr -->
        <xsl:param name="element" as="xs:string"/>
        <!-- mmgr oder mm lokal in $direktes-elternelement -->

        <xsl:if test="$debug and $verbose">
            <xsl:message>
                <xsl:value-of
                    select="concat('-- verarbeite ', $direktes-elternelement, '/', $element, ' --')"
                />
            </xsl:message>
        </xsl:if>

        <xsl:choose>
            <xsl:when
                test="
                    $direktes-elternelement = 'ALGID' or
                    $direktes-elternelement = 'ASID' or
                    $direktes-elternelement = 'DGUID' or
                    $direktes-elternelement = 'EAID' or
                    $direktes-elternelement = 'FAID' or
                    $direktes-elternelement = 'HWKID' or
                    $direktes-elternelement = 'IHKID' or
                    $direktes-elternelement = 'ISID' or
                    $direktes-elternelement = 'LUEID' or
                    $direktes-elternelement = 'RGID' or
                    $direktes-elternelement = 'STAID' or
                    $direktes-elternelement = 'ZVID'
                    ">
                <xsl:choose>
                    <xsl:when test="$element = 'SatzErstellung'">1</xsl:when>
                    <xsl:when test="$element = 'Autor'">2</xsl:when>
                    <xsl:when test="$element = 'IstKorrektur'">3</xsl:when>
                    <xsl:when test="$element = 'KorrekturGrundSchluessel'">4</xsl:when>
                    <xsl:when test="$element = 'GemeindeSchluessel'">5</xsl:when>
                    <xsl:when test="$element = 'GemeindeMeldungNr'">7</xsl:when>
                    <xsl:when test="$element = 'GewerbebetriebID'">8</xsl:when>
                    <xsl:when test="$element = 'UUID'">9</xsl:when>
                    <xsl:when test="$element = 'Gewerbe'">10</xsl:when>
                    <xsl:when test="$element = 'GueltigAb'">11</xsl:when>
                    <xsl:when test="$element = 'DatumMeldung'">12</xsl:when>
                    <xsl:when test="$element = 'TaetigkeitenGesamt'">13</xsl:when>
                    <xsl:when test="$element = 'SpezifischeInformation'">15</xsl:when>

                    <!-- Anmeldung -->
                    <xsl:when test="$element = 'FrueheresGewerbe'">16</xsl:when>
                    <xsl:when test="$element = 'GrundAnmeldungSchluessel'">17</xsl:when>
                    <xsl:when test="$element = 'VerdachtsmomenteText'">18</xsl:when>
                    <xsl:when test="$element = 'Verdachtsmomente'">19</xsl:when>

                    <!-- Ummeldung -->
                    <xsl:when test="$element = 'GruendeUmmeldung'">17</xsl:when>
                    <xsl:when test="$element = 'AnlaesseUmmeldung'">18</xsl:when>

                    <!-- Abmeldung -->
                    <xsl:when test="$element = 'KuenftigesGewerbe'">16</xsl:when>
                    <xsl:when test="$element = 'GrundAbmeldungSchluessel'">17</xsl:when>
                    <xsl:when test="$element = 'UrsacheAbmeldung'">18</xsl:when>
                    <xsl:when test="$element = 'UrsacheAbmeldungSchluessel'">19</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'Autor'">
                <xsl:choose>
                    <xsl:when test="$element = 'BerichtseinheitID'">1</xsl:when>
                    <xsl:when test="$element = 'NameOrganisation'">2</xsl:when>
                    <xsl:when test="$element = 'TelefonVorwahl'">3</xsl:when>
                    <xsl:when test="$element = 'TelefonNummer'">4</xsl:when>
                    <xsl:when test="$element = 'FaxVorwahl'">5</xsl:when>
                    <xsl:when test="$element = 'FaxNummer'">6</xsl:when>
                    <xsl:when test="$element = 'EMailAdresse'">7</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'Aufenthaltsgenehmigung'">
                <xsl:choose>
                    <xsl:when test="$element = 'Status'"/>
                    <xsl:when test="$element = 'AusgestelltAm'"/>
                    <xsl:when test="$element = 'AusgestelltVonName'"/>
                    <xsl:when test="$element = 'AusgestelltVonPLZ'"/>
                    <xsl:when test="$element = 'AusgestelltVonOrt'"/>
                    <xsl:when test="$element = 'Auflagen'"/>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'BeteiligteFirma'">
                <xsl:choose>
                    <xsl:when test="$element = 'IstBeteiligtAn'">1</xsl:when>
                    <xsl:when test="$element = 'Betrieb'">2</xsl:when>
                    <xsl:when test="$element = 'BeteiligtePerson'">3</xsl:when>
                    <xsl:when test="$element = 'Anschrift'">4</xsl:when>
                    <xsl:when test="$element = 'Handwerkskarte'">5</xsl:when>
                    <xsl:when test="$element = 'Erlaubnis'">6</xsl:when>
                    <xsl:when test="$element = 'EintrittsdatumFirma'">7</xsl:when>
                    <xsl:when test="$element = 'AustrittsdatumFirma'">8</xsl:when>
                    <xsl:when test="$element = 'AnzahlBeteiligte'">9</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'Erlaubnis'">
                <xsl:choose>
                    <xsl:when test="$element = 'Status'">1</xsl:when>
                    <xsl:when test="$element = 'AusgestelltAm'">2</xsl:when>
                    <xsl:when test="$element = 'AusgestelltVonName'">3</xsl:when>
                    <xsl:when test="$element = 'AusgestelltVonPLZ'">4</xsl:when>
                    <xsl:when test="$element = 'AusgestelltVonOrt'">5</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'GruendeUmmeldung'">
                <xsl:choose>
                    <xsl:when test="$element = 'GrundUmmeldungSchluessel'">1</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'Handwerkskarte'">
                <xsl:choose>
                    <xsl:when test="$element = 'Status'">1</xsl:when>
                    <xsl:when test="$element = 'AusgestelltAm'">2</xsl:when>
                    <xsl:when test="$element = 'AusgestelltVonName'">3</xsl:when>
                    <xsl:when test="$element = 'AusgestelltVonPLZ'">4</xsl:when>
                    <xsl:when test="$element = 'AusgestelltVonOrt'">5</xsl:when>
                    <xsl:when test="$element = 'AusgestelltVonSchluessel'">6</xsl:when>
                    <xsl:when test="$element = 'ArtHandwerkskarte'">7</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'Hauptniederlassung'">
                <xsl:choose>
                    <xsl:when test="$element = 'Anschrift'">1</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'KuenftigesGewerbe'">
                <xsl:choose>
                    <xsl:when test="$element = 'NameFirmaGewerbetreibender'">1</xsl:when>
                    <xsl:when test="$element = 'Anschrift'">2</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'Staatangehoerigkeiten'">
                <xsl:choose>
                    <xsl:when test="$element = 'Staatsangehoerigkeit'">1</xsl:when>
                    <xsl:when test="$element = 'StaatsangehoerigkeitSchluessel'">2</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'Verdachtsmomente'">
                <xsl:choose>
                    <xsl:when test="$element = 'VerdachtsmomentSchluessel'">1</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'VertretungsberechtigtePerson'">
                <xsl:choose>
                    <xsl:when test="$element = 'Familienname'">1</xsl:when>
                    <xsl:when test="$element = 'Vorname'">2</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'WeitereNeuAusgeuebteTaetigkeiten'">
                <xsl:choose>
                    <xsl:when test="$element = 'TaetigkeitenZusammengefasst'">1</xsl:when>
                    <xsl:when test="$element = 'Taetigkeit'">2</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'TaetigkeitenGesamt'">
                <xsl:choose>
                    <xsl:when test="$element = 'Schwerpunkt'">1</xsl:when>
                    <xsl:when test="$element = 'WeitereTaetigkeiten'">2</xsl:when>
                    <xsl:when test="$element = 'WeitereNeuAusgeuebteTaetigkeiten'">3</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'Schwerpunkt'">
                <xsl:choose>
                    <xsl:when test="$element = 'TaetigkeitSchwerpunkt'">1</xsl:when>
                    <xsl:when test="$element = 'TaetigkeitSchluessel'">2</xsl:when>
                    <xsl:when test="$element = 'Stichwort'">3</xsl:when>
                    <xsl:when test="$element = 'BeginnTaetigkeit'">4</xsl:when>
                    <xsl:when test="$element = 'EndeTaetigkeit'">5</xsl:when>
                    <xsl:when test="$element = 'NeuAusgeuebt'">6</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'WeitereTaetigkeiten'">
                <xsl:choose>
                    <xsl:when test="$element = 'TaetigkeitenZusammengefasst'">1</xsl:when>
                    <xsl:when test="$element = 'Taetigkeit'">2</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'Taetigkeit'">
                <xsl:choose>
                    <xsl:when test="$element = 'TaetigkeitEinzeln'">1</xsl:when>
                    <xsl:when test="$element = 'TaetigkeitSchluessel'">2</xsl:when>
                    <xsl:when test="$element = 'Stichwort'">3</xsl:when>
                    <xsl:when test="$element = 'BeginnTaetigkeit'">4</xsl:when>
                    <xsl:when test="$element = 'EndeTaetigkeit'">4</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'SpezifischeInformation'">
                <xsl:choose>
                    <xsl:when test="$element = 'Begriff'">1</xsl:when>
                    <xsl:when test="$element = 'Inhalt'">2</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'FrueheresGewerbe'">
                <xsl:choose>
                    <xsl:when test="$element = 'NameFirmaGewerbetreibender'">1</xsl:when>
                    <xsl:when test="$element = 'Anschrift'">2</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'Anschrift'">
                <xsl:choose>
                    <xsl:when test="$element = 'Strasse'">1</xsl:when>
                    <xsl:when test="$element = 'Strassenschluessel'">2</xsl:when>
                    <xsl:when test="$element = 'HausNrVon'">3</xsl:when>
                    <xsl:when test="$element = 'HausNrBis'">4</xsl:when>
                    <xsl:when test="$element = 'AnschriftZusatz'">5</xsl:when>
                    <xsl:when test="$element = 'Ort'">6</xsl:when>
                    <xsl:when test="$element = 'Ortsteil'">7</xsl:when>
                    <xsl:when test="$element = 'PLZ'">8</xsl:when>
                    <xsl:when test="$element = 'NationSchluessel'">9</xsl:when>
                    <xsl:when test="$element = 'NATSchluesselVerzeichnis'">10</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'Gewerbe'">
                <xsl:choose>
                    <xsl:when test="$element = 'Betrieb'">1</xsl:when>
                    <xsl:when test="$element = 'AnzahlBeteiligte'">2</xsl:when>
                    <xsl:when test="$element = 'AnzahlVollzeitBeschaeftigte'">3</xsl:when>
                    <xsl:when test="$element = 'AnzahlTeilzeitBeschaeftigte'">4</xsl:when>
                    <xsl:when test="$element = 'BetriebArtIndustrie'">5</xsl:when>
                    <xsl:when test="$element = 'BetriebArtHandwerk'">6</xsl:when>
                    <xsl:when test="$element = 'BetriebArtHandel'">7</xsl:when>
                    <xsl:when test="$element = 'BetriebArtSonstiges'">8</xsl:when>
                    <xsl:when test="$element = 'NiederlassungArt'">9</xsl:when>
                    <xsl:when test="$element = 'IstReisegewerbe'">10</xsl:when>
                    <xsl:when test="$element = 'IstNebenerwerb'">11</xsl:when>
                    <xsl:when test="$element = 'Handwerkskarte'">12</xsl:when>
                    <xsl:when test="$element = 'Erlaubnis'">13</xsl:when>
                    <xsl:when test="$element = 'VertretungsberechtigtePerson'">14</xsl:when>
                    <xsl:when test="$element = 'Anschrift'">15</xsl:when>
                    <xsl:when test="$element = 'Hauptniederlassung'">16</xsl:when>
                    <xsl:when test="$element = 'BeteiligtePerson'">17</xsl:when>
                    <xsl:when test="$element = 'BeteiligteFirma'">18</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'Betrieb'">
                <xsl:choose>
                    <xsl:when test="$element = 'EingetragenerName'">1</xsl:when>
                    <xsl:when test="$element = 'BetriebsstaettenBezeichnung'">2</xsl:when>
                    <xsl:when test="$element = 'EintragungNr'">3</xsl:when>
                    <xsl:when test="$element = 'EintragungNrSonstige'">4</xsl:when>
                    <xsl:when test="$element = 'EintragungOrt'">5</xsl:when>
                    <xsl:when test="$element = 'EintragungPLZ'">6</xsl:when>
                    <xsl:when test="$element = 'EintragungArt'">7</xsl:when>
                    <xsl:when test="$element = 'GerichtSchluessel'">8</xsl:when>
                    <xsl:when test="$element = 'RechtsformSchluessel'">9</xsl:when>
                    <xsl:when test="$element = 'GbRGesellschafter'">10</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:when test="$direktes-elternelement = 'BeteiligtePerson'">
                <xsl:choose>
                    <xsl:when test="$element = 'PersonFunktion'">1</xsl:when>
                    <xsl:when test="$element = 'Doktorgrad'">2</xsl:when>
                    <xsl:when test="$element = 'Familienname'">3</xsl:when>
                    <xsl:when test="$element = 'Vorname'">4</xsl:when>
                    <xsl:when test="$element = 'NamenZusatz'">5</xsl:when>
                    <xsl:when test="$element = 'Geschlecht'">6</xsl:when>
                    <xsl:when test="$element = 'Geburtsname'">7</xsl:when>
                    <xsl:when test="$element = 'Geburtsdatum'">8</xsl:when>
                    <xsl:when test="$element = 'Geburtsort'">9</xsl:when>
                    <xsl:when test="$element = 'Geburtsland'">10</xsl:when>
                    <xsl:when test="$element = 'IstNebenerwerb'">11</xsl:when>
                    <xsl:when test="$element = 'EintrittsdatumPerson'">12</xsl:when>
                    <xsl:when test="$element = 'AustrittsdatumPerson'">13</xsl:when>
                    <xsl:when test="$element = 'Staatangehoerigkeiten'">14</xsl:when>
                    <xsl:when test="$element = 'Anschrift'">15</xsl:when>
                    <xsl:when test="$element = 'Handwerkskarte'">16</xsl:when>
                    <xsl:when test="$element = 'Erlaubnis'">17</xsl:when>
                    <xsl:when test="$element = 'Aufenthaltsgenehmigung'">18</xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="xga:element-nicht-gefunden-message">
                            <xsl:with-param name="direktes-elternelement"
                                select="$direktes-elternelement" as="xs:string"/>
                            <xsl:with-param name="element" select="$element" as="xs:string"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>

            <xsl:otherwise>
                <xsl:call-template name="xga:element-nicht-gefunden-message">
                    <xsl:with-param name="direktes-elternelement" select="$direktes-elternelement"
                        as="xs:string"/>
                    <xsl:with-param name="element" select="$element" as="xs:string"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <!-- Hilfsfunktion fürs Debugging, verwendet von xga:local-position-for-element -->
    <xsl:template name="xga:element-nicht-gefunden-message">
        <xsl:param name="direktes-elternelement" as="xs:string"/>
        <xsl:param name="element" as="xs:string"/>
        <xsl:if test="$debug">
            <xsl:message>
                <xsl:value-of
                    select="concat('Bei Sortierung nicht gefunden: ', $direktes-elternelement, '/', $element)"
                />
            </xsl:message>
        </xsl:if>
    </xsl:template>

    <!-- *****************************************************************************************************************************
        Ab hier: Sonderfälle, bei denen die Zielstruktur jenseits der Transposition [datml:mm name='Foo'] -> [xga:foo] weiter abweicht
        ****************************************************************************************************************************** -->

    <!-- Ausschlussliste: Einige Elemente sollen nicht in der Zielstruktur erzeugt werden -->
    <xsl:template
        match="
            datml:mm[@name = 'Satzart' or
            @name = 'MeldungArt' or
            @name = 'AGSSchluesselVerzeichnis' or
            @name = 'SGSSchluesselVerzeichnis' or
            @name = 'BetriebArtIndustrie' or
            @name = 'BetriebArtHandwerk' or
            @name = 'BetriebArtHandel' or
            @name = 'BetriebArtSonstiges' or
            @name = 'TelefonNummer' or
            @name = 'TelefonVorwahl' or
            @name = 'FaxNummer' or
            @name = 'FaxVorwahl' or
            @name = 'WebAdresse' or
            @name = 'EMailAdresse' or
            @name = 'NamenZusatz' or
            @name = 'GemeindeZusatz'
            ]">
        <xsl:if test="$debug">
            <xsl:comment>
                <xsl:value-of select="concat('Der Typ [', ./@name, '] ist in XGA 2.0 in dieser Form obsolet.')"/>
            </xsl:comment>
            <xsl:message>
                <xsl:value-of
                    select="concat('|| ', ./@name, ': Der Typ wurde nicht (oder an anderer Stelle) in die Zielstruktur geschrieben, da er in XGA 2.0 obsolet ist.')"
                />
            </xsl:message>
        </xsl:if>
    </xsl:template>

    <!-- Hilfsfunktion: Erzeuge "xga:fooBar" aus "FooBar" -->
    <xsl:function name="xga:typ-schreibweise">
        <xsl:param name="quelltyp" as="xs:string"/>
        <xsl:variable name="zieltyp" as="xs:string">
            <xsl:value-of
                select="concat('xga:', lower-case(substring($quelltyp, 1, 1)), substring($quelltyp, 2))"
            />
        </xsl:variable>
        <xsl:value-of select="$zieltyp"/>
    </xsl:function>

    <!-- Benanntes Hilfstemplate: Wandelt den Datumsinhalt des Typs von "TTMMJJJJ" zu "JJJJ-MM-TT" -->
    <xsl:function name="xga:sql-datum" as="xs:string">
        <xsl:param name="quelltyp" as="node()"/>
        <xsl:variable name="jahr" as="xs:string" select="substring($quelltyp/datml:wert, 5, 4)"/>
        <xsl:variable name="monat" as="xs:string" select="substring($quelltyp/datml:wert, 3, 2)"/>
        <xsl:variable name="tag" as="xs:string" select="substring($quelltyp/datml:wert, 1, 2)"/>
        <xsl:value-of select="concat($jahr, '-', $monat, '-', $tag)"/>
    </xsl:function>

    <!-- Hilfsfunktion: Erzeuge den DatML-RAW-Typ als XGA-Typ, dessen Inhalt jedoch von einem <code>-Tag umgeben ist -->
    <xsl:function name="xga:fuege-kindelement-code-hinzu">
        <xsl:param name="quelltyp" as="node()"/>
        <xsl:element name="{xga:typ-schreibweise($quelltyp/@name)}">
            <code>
                <xsl:value-of select="$quelltyp/datml:wert"/>
            </code>
        </xsl:element>
    </xsl:function>

    <!-- Fasse die bisherigen Elemente /BetriebArt* in neuem Elternelement gewerbe/betriebArt zusammen;
         erzeuge außerdem ein eigenes Kindelement erreichbarekeit mit den Angaben zur Kommunikation -->
    <xsl:template match="datml:mmgr[@name = 'Gewerbe']">
        <xga:gewerbe>
            <xsl:apply-templates
                select="
                    (datml:mm | datml:mmgr)[
                    xga:lokale-position-fuer-element('Gewerbe', @name) lt xga:lokale-position-fuer-element('Gewerbe', 'BetriebArtIndustrie')]">
                <xsl:sort select="xga:lokale-position-fuer-element('Gewerbe', @name)"/>
            </xsl:apply-templates>
            <xsl:if
                test="datml:mm[@name = ('BetriebArtIndustrie', 'BetriebArtHandwerk', 'BetriebArtHandel', 'BetriebArtSonstiges')]">
                <xga:betriebArt>
                    <xsl:apply-templates
                        select="
                            (datml:mm | datml:mmgr)[
                            xga:lokale-position-fuer-element('Gewerbe', @name) ge xga:lokale-position-fuer-element('Gewerbe', 'BetriebArtIndustrie') and
                            xga:lokale-position-fuer-element('Gewerbe', @name) le xga:lokale-position-fuer-element('Gewerbe', 'BetriebArtSonstiges')]"
                        mode="xga:betriebart"/>
                </xga:betriebArt>
            </xsl:if>

            <!-- Erzeuge alles nach "betriebArt" bis einschließlich "anschrift" -->
            <xsl:apply-templates
                select="
                    (datml:mm | datml:mmgr)[
                    xga:lokale-position-fuer-element('Gewerbe', @name) gt xga:lokale-position-fuer-element('Gewerbe', 'BetriebArtSonstiges') and
                    xga:lokale-position-fuer-element('Gewerbe', @name) lt xga:lokale-position-fuer-element('Gewerbe', 'Hauptniederlassung')]">
                <xsl:sort select="xga:lokale-position-fuer-element('Gewerbe', @name)"/>
            </xsl:apply-templates>

            <!-- Erzeuge den Rest von Gewerbe (alles nach "erreichbarkeit") -->
            <xsl:apply-templates
                select="
                    (datml:mm | datml:mmgr)[
                    xga:lokale-position-fuer-element('Gewerbe', @name) gt xga:lokale-position-fuer-element('Gewerbe', 'Anschrift')]">
                <xsl:sort select="xga:lokale-position-fuer-element('Gewerbe', @name)"/>
            </xsl:apply-templates>

            <!-- beteiligungOeffentlicheHand [1] -->
            <xsl:call-template name="erzeuge-beteiligungOeffentlicheHand"/>
        </xga:gewerbe>
    </xsl:template>

    <!-- XGA-477: Erreichbarkeitsdetails werden aus Anschrift herausgezogen -->
    <xsl:template match="datml:mmgr[@name = 'Anschrift']">
        <!-- Die herauszuziehenden Kindelemente werden mittels Ausschlussliste am Anfang des Skripts unterdrückt, sodass hier nur die in Anschrift 
             tatsächlich noch erwünschten Elemente erzeugt werden -->
        <xga:anschrift>
            <xsl:apply-templates/>
        </xga:anschrift>

        <!-- Anschließend separate Angabe der Erreichbarkeitsdetails gemäß CR -->
        <xsl:variable name="TelefonVorwahl" as="xs:string?"
            select="datml:mm[@name = 'TelefonVorwahl']/datml:wert/text()"/>
        <xsl:variable name="TelefonNummer" as="xs:string?"
            select="datml:mm[@name = 'TelefonNummer']/datml:wert/text()"/>
        <xsl:variable name="FaxVorwahl" as="xs:string?"
            select="datml:mm[@name = 'FaxVorwahl']/datml:wert/text()"/>
        <xsl:variable name="FaxNummer" as="xs:string?"
            select="datml:mm[@name = 'FaxNummer']/datml:wert/text()"/>
        <xsl:variable name="WebAdresse" as="xs:string?"
            select="datml:mm[@name = 'WebAdresse']/datml:wert/text()"/>
        <xsl:variable name="EMailAdresse" as="xs:string?"
            select="datml:mm[@name = 'EMailAdresse']/datml:wert/text()"/>

        <xsl:if test="$EMailAdresse">
            <xga:erreichbarkeit>
                <xga:kanal>
                    <code>01<!--E-Mail--></code>
                </xga:kanal>
                <xga:kennung>
                    <xsl:value-of select="$EMailAdresse"/>
                </xga:kennung>
            </xga:erreichbarkeit>
        </xsl:if>

        <xsl:if test="$TelefonVorwahl or $TelefonNummer">
            <xga:erreichbarkeit>
                <xga:kanal>
                    <code>02<!--Telefon Festnetz--></code>
                </xga:kanal>
                <xga:kennung>
                    <!-- string-join konkateniert zwei Strings unter Verwendnung des in arg2 angegebenen Separators; 
                            gibt es nur einen String, wird nur dieser (ohne Separator) ausgegeben! -->
                    <xsl:value-of select="string-join(($TelefonVorwahl, $TelefonNummer), '-')"/>
                </xga:kennung>
            </xga:erreichbarkeit>
        </xsl:if>

        <xsl:if test="$FaxVorwahl or $FaxNummer">
            <xga:erreichbarkeit>
                <xga:kanal>
                    <code>04<!--Fax--></code>
                </xga:kanal>
                <xga:kennung>
                    <xsl:value-of select="string-join(($FaxVorwahl, $FaxNummer), '-')"/>
                </xga:kennung>
            </xga:erreichbarkeit>
        </xsl:if>

        <xsl:if test="$WebAdresse">
            <xga:erreichbarkeit>
                <xga:kanal>
                    <code>09<!--Web--></code>
                </xga:kanal>
                <xga:kennung>
                    <xsl:value-of select="$WebAdresse"/>
                </xga:kennung>
            </xga:erreichbarkeit>
        </xsl:if>
    </xsl:template>

    <!-- XGA-464: Erzeuge "beteiligungOeffentlicheHand"; da es dieses Feld in 1.3 noch nicht gab, haben gemäß EG-Entscheidung (vgl. Kommentar im CR) 
         die Alt-Fälle hier stets ein "nein" (d.i. "0") -->
    <xsl:template name="erzeuge-beteiligungOeffentlicheHand">
        <xsl:choose>
            <xsl:when
                test="ancestor::datml:satz/datml:mm[@name = 'Satzart']/datml:wert = ('DGUID', 'ALGID')">
                <xga:beteiligungOeffentlicheHand>
                    <code>0</code>
                </xga:beteiligungOeffentlicheHand>
            </xsl:when>
            <xsl:otherwise><!-- nichts ausgeben --></xsl:otherwise>
        </xsl:choose>

    </xsl:template>

    <!-- XGA-511: Umbenennung BetriebsstaettenBezeichnung in geschaeftsbezeichnung -->
    <xsl:template match="datml:mm[@name = 'BetriebsstaettenBezeichnung']">
        <xga:geschaeftsbezeichnung>
            <xsl:value-of select="datml:wert"/>
        </xga:geschaeftsbezeichnung>
    </xsl:template>

    <!-- Ausschluss von Aufenthaltgenehmigung bei den Satzarten IHK und HWK (XGA-448) und, bei allen anderen:
         Umbenennung Aufenthaltsgenehmigung in Aufenthaltstitel (XGA-507, 508) -->
    <xsl:template match="datml:mmgr[@name = 'Aufenthaltsgenehmigung']">
        <xsl:choose>

            <!-- XGA-448: Keine Übermittlung bei IHK und HWK -->
            <xsl:when
                test="ancestor::datml:satz/datml:mm[@name = 'Satzart']/datml:wert = ('IHKID', 'HWKID')">
                <!-- nichts ausgeben -->
            </xsl:when>

            <!-- XGA-507/508: Aus ~genehmigung wird ~titel -->
            <xsl:otherwise>
                <xga:aufenthaltstitel>
                    <!-- Die Kindelemente bleiben unverändert-->
                    <xsl:apply-templates/>
                </xga:aufenthaltstitel>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="$debug">
            <xsl:message>Aufenthaltsgenehmigung ausgelassen gemäß XGA-448</xsl:message>
        </xsl:if>
    </xsl:template>

    <!-- Ausschluss von EintragungNrSonstige bei der Satzart EA gemäß Abschnitt "I.6.2.9.1 Nachrichtenspezifische Einschränkungen" der 2.0-Spezifikation -->
    <xsl:template
        match="datml:mm[@name = 'EintragungNrSonstige' and ancestor::datml:satz/datml:mm[@name = 'Satzart']/datml:wert = 'EAID']">
        <!--nichts erzeugen!-->
    </xsl:template>

    <!-- XGA-442: Zusätzliches Element xga:bisherigeUnfallversicherung bei den Satzarten DGUID udn ALGID.
         Es wird behelfsweise stets nach "taetigkeitenGesamt" eingefügt, weil es kein genuines Match-Element in 1.3. gibt -->
    <xsl:template match="datml:mmgr[@name = 'TaetigkeitenGesamt']">
        <!-- erstmal xga:taetigkeitenGesamt erzeugen -->
        <xga:taetigkeitenGesamt>
            <xsl:apply-templates select="datml:mmgr[@name = 'Schwerpunkt']"/>
            <xsl:apply-templates select="datml:mmgr[@name = 'WeitereTaetigkeiten']"/>
            <xsl:apply-templates select="datml:mmgr[@name = 'WeitereNeuAusgeuebteTaetigkeiten']"/>
        </xga:taetigkeitenGesamt>

        <!-- Wegen SCH-0094, SCH-0095 und SCH-0096: Bei den Satzarten DGUID und ALGID muss (außer bei Neugründungen, wo 
             es unzulässig ist) die bisherige Unfallversicherung angegeben werden -->
        <xsl:if
            test="ancestor::datml:satz/datml:mm[@name = 'Satzart']/datml:wert = ('DGUID', 'ALGID')">
            <xsl:choose>
                <!-- Im Falle einer Neugründung: Keine weitere Angabe -->
                <xsl:when
                    test="ancestor::datml:satz/datml:mm[@name = 'GrundAnmeldungSchluessel']/datml:wert = '01'">
                    <!-- nichts ausgeben -->
                </xsl:when>

                <!-- In allen anderen Fällen: Angabe der bisherigen Unfallversicherung -->
                <xsl:otherwise>
                    <xga:bisherigeUnfallversicherung>
                        <xga:traeger listVersionID="1">
                            <code>
                                <xsl:value-of select="1234"/>
                                <xsl:comment>Dummy-Wert, da es in XGA-1.3 keine korrespondiere Angabe als Quelle gibt!</xsl:comment>
                            </code>
                        </xga:traeger>
                        <xga:mitgliedsnummer>
                            <xsl:value-of select="1234"/>
                            <xsl:comment>Dummy-Wert, da es in XGA-1.3 keine korrespondiere Angabe als Quelle gibt!</xsl:comment>
                        </xga:mitgliedsnummer>
                    </xga:bisherigeUnfallversicherung>
                </xsl:otherwise>
            </xsl:choose>

        </xsl:if>
    </xsl:template>

    <!-- Quelltyp benötigt jeweils ein zusätzliches Kindelement <code> -->
    <xsl:template
        match="
            datml:mm[@name = ('PersonFunktion',
            'NeuAusgeuebt',
            'GrundAnmeldungSchluessel',
            'GrundUmmeldungSchluessel',
            'GrundAbmeldungSchluessel',
            'UrsacheAbmeldungSchluessel',
            'NiederlassungArt',
            'RechtsformSchluessel',
            'EintragungArt',
            'IstKorrektur',
            'KorrekturGrundSchluessel',
            'AusgestelltVonSchluessel',
            'VerdachtsmomentSchluessel',
            'IstReisegewerbe',
            'IstNebenerwerb'
            )]">
        <xsl:sequence select="xga:fuege-kindelement-code-hinzu(.)"/>
    </xsl:template>

    <xsl:template match="datml:mm[@name = 'Status']">
        <xga:status>
            <code>
                <xsl:value-of select="datml:wert"/>
            </code>
        </xga:status>
    </xsl:template>

    <!-- Das Datumsformat des Quelltyp-Inhalts muss in ein SQL-Datum umgewandelt werden -->
    <xsl:template
        match="
            datml:mm[@name = ('GueltigAb',
            'DatumMeldung',
            'AusgestelltAm'
            )]">
        <xsl:element name="{xga:typ-schreibweise(@name)}">
            <xsl:value-of select="xga:sql-datum(.)"/>
        </xsl:element>
    </xsl:template>

    <!-- xga:satzErstellung: Umwandlung des Datumsformats in {SQL-Datum}T{HH:mm:SS}.0 -->
    <xsl:template match="datml:mm[@name = 'SatzErstellung']">
        <xsl:variable name="jahr" as="xs:string" select="substring(datml:wert, 1, 4)"/>
        <xsl:variable name="monat" as="xs:string" select="substring(datml:wert, 5, 2)"/>
        <xsl:variable name="tag" as="xs:string" select="substring(datml:wert, 7, 2)"/>
        <xsl:variable name="stunde" as="xs:string" select="substring(datml:wert, 9, 2)"/>
        <xsl:variable name="minute" as="xs:string" select="substring(datml:wert, 11, 2)"/>
        <xsl:variable name="sekunde" as="xs:string" select="substring(datml:wert, 13, 2)"/>
        <xga:satzErstellung>
            <xsl:value-of
                select="concat($jahr, '-', $monat, '-', $tag, 'T', $stunde, ':', $minute, ':', $sekunde, '.0')"
            />
        </xga:satzErstellung>
    </xsl:template>

    <!-- Der Quelltyp muss im Ziel vollständig in Kleinbuchstaben und ohne Kindelement abgebildet werden -->
    <xsl:template match="
            datml:mm[@name = ('UUID', 'PLZ')]">
        <xsl:element name="{concat('xga:', lower-case(@name))}">
            <xsl:value-of select="datml:wert"/>
        </xsl:element>
    </xsl:template>

    <!-- NATSchluesselverzeichnis: Zieltyp umbenennen und ohne Kindelement abbilden -->
    <xsl:template match="datml:mm[@name = 'NATSchluesselVerzeichnis']">
        <xga:natSchluesselVerzeichnis>
            <xsl:value-of select="datml:wert"/>
        </xga:natSchluesselVerzeichnis>
    </xsl:template>

    <!-- Transposition der Betriebarten von BetriebArtFoo (= 1.3) zu betriebArt/foo (= 2.0) -->
    <xsl:template
        match="datml:mm[@name = ('BetriebArtIndustrie', 'BetriebArtHandwerk', 'BetriebArtHandel', 'BetriebArtSonstiges')]"
        mode="xga:betriebart">
        <xsl:variable name="neuer-name" select="substring-after(./@name, 'BetriebArt')"
            as="xs:string"/>
        <xsl:element
            name="{concat('xga:', lower-case(substring($neuer-name, 1, 1)),
                                   substring($neuer-name, 2))
                           }">
            <code>
                <xsl:value-of select="./datml:wert"/>
            </code>
        </xsl:element>
    </xsl:template>

    <!-- xga:gemeindeSchluessel: listeVersionID und zusätzliches Kindelement 'code' hinzufügen -->
    <xsl:template match="datml:mm[@name = 'GemeindeSchluessel']">
        <xsl:variable name="datum" as="xs:string"
            select="substring-after(../datml:mm[@name = 'AGSSchluesselVerzeichnis']/datml:wert, 'ags_')"/>
        <xga:gemeindeSchluessel listVersionID="{$datum}">
            <code>
                <xsl:value-of select="datml:wert"/>
            </code>
        </xga:gemeindeSchluessel>
    </xsl:template>

    <!-- xga:StaatsangehoerigkeitSchluessel: listVersionID und zusätzliches Kindelement 'code' -->
    <xsl:template match="datml:mm[@name = 'StaatsangehoerigkeitSchluessel']">
        <xsl:variable name="datum" as="xs:string"
            select="substring-after(../../datml:mm[@name = 'SGSSchluesselVerzeichnis']/datml:wert, 'staatsangehoerigkeit_')"/>
        <xga:staatsangehoerigkeitSchluessel listVersionID="{$datum}">
            <code>
                <xsl:value-of select="datml:wert"/>
            </code>
        </xga:staatsangehoerigkeitSchluessel>
    </xsl:template>

    <!-- xga:satz/xga:autor ist vom typ Behoerde.Erreichbar, d.h. er entspricht strukturell dem Autor im Nachrichtenkopf. -->
    <xsl:template match="datml:mmgr[@name = 'Autor']">
        <xsl:param name="satz-nummer" as="xs:integer" required="yes"/>
        <xga:autor>
            <!-- Datenquelle: Satzebene. -->
            <xsl:call-template name="xga:behoerdenkennung-autor-satz">
                <xsl:with-param name="satz-nummer" select="$satz-nummer"/>
            </xsl:call-template>

            <!-- Datenquelle: Entweder vollständig Satzebene, oder vollständig Nachrichtenkopf. -->
            <xsl:call-template name="xga:erreichbarkeit-autor-satz">
                <xsl:with-param name="satz-nummer" as="xs:integer" select="$satz-nummer"/>
            </xsl:call-template>

            <!-- Datenquelle: Nachrichtenkopf (absender/identififkation/adresse), da auf Satzebene nicht vorhanden. -->
            <xsl:call-template name="xga:anschrift-autor"/>

            <!-- Datenquelle: Satzebene. -->
            <xsl:call-template name="xga:behoerdenname-autor-satz">
                <xsl:with-param name="satz-nummer" as="xs:integer" select="$satz-nummer"/>
            </xsl:call-template>
        </xga:autor>
    </xsl:template>

    <!-- Diese Angabe ist immer AGS-scharf, d.h. hier steht immer der AGS der Betriebsstätte des Satzes -->
    <xsl:template name="xga:behoerdenkennung-autor-satz">
        <xsl:param name="satz-nummer" as="xs:integer" required="yes"/>
        <xga:behoerdenkennung>
            <!-- auf Satzebene immer ein AGS, d.h. konstant "MELDID" -->
            <xga:praefix listVersionID="7">
                <code>MELDID</code>
            </xga:praefix>
            <xsl:variable name="urn" as="xs:string"
                select="'urn:de:bund:destatis:bevoelkerungsstatistik:schluessel:ags'"/>
            <xga:kennung listURI="{$urn}" listVersionID="{xga:listVersionID($urn)}">
                <xsl:variable name="ags-schluessel" as="xs:string?"
                    select="/datml:DatML-RAW-D/datml:nachricht/datml:datensegment/datml:satz[$satz-nummer]/datml:mm[@name = 'GemeindeSchluessel']/datml:wert"/>
                <code>
                    <xsl:value-of select="$ags-schluessel"/>
                </code>
                <xsl:if test="$code-name-hinzufuegen and $ags-schluessel">
                    <name>
                        <xsl:value-of
                            select="xga:codelisten-wert($urn, $ags-schluessel, 'Bezeichnung')"/>
                    </name>
                </xsl:if>
            </xga:kennung>
        </xga:behoerdenkennung>
    </xsl:template>

    <!-- Angaben entweder exklusiv auf Satzebene, oder exklusiv aus Kopf -->
    <xsl:template name="xga:erreichbarkeit-autor-satz">
        <xsl:param name="satz-nummer" as="xs:integer" required="yes"/>

        <!-- Grundgedanke: Die Datenquellen (Satz vs. Nachrichtenkopf) für die Angaben zum Autor auf Satzebene sollen nicht "gemischt" werden. 
             In 2.0 muss zumindest ein Erreichbarkeitskanal angegeben werden, die weiteren Angaben zur Erreichbarkeit des Autors sind dort optional
             (weil autor/erreichbarkeit : Behoerde.Erreichbar [1..n] gilt). Ist mindestens die Telefonnummer oder E-Mail-Adresse des Autors auf Satzebene
             angegeben (andere nach 2.0 transponierbare Erreichbarkeitskanäle kennt 1.3 
             ), werden diese Informatiomen verwendet. Anderfalls
             wird vollständig auf die vorhandenen Erreichbarkeitsdaten des Autors auf Kopfebene zurückgegriffen.             
        -->

        <!-- Telefon-Vorwahl + Nummer aus Satz extrahieren (kann leer sein) -->
        <xsl:variable name="telefon-vorwahl-satz" as="xs:string?"
            select="
                /datml:DatML-RAW-D/datml:nachricht/datml:datensegment/datml:satz[$satz-nummer]/
                datml:mmgr[@name = 'Autor']/datml:mm[@name = 'TelefonVorwahl']/datml:wert
                "/>

        <xsl:variable name="telefon-nummer-satz" as="xs:string?"
            select="
                /datml:DatML-RAW-D/datml:nachricht/datml:datensegment/datml:satz[$satz-nummer]/
                datml:mmgr[@name = 'Autor']/datml:mm[@name = 'TelefonNummer']/datml:wert
                "/>

        <!-- E-Mail aus Satz extrahieren (kann leer sein) -->
        <xsl:variable name="email-satz" as="xs:string?"
            select="
                /datml:DatML-RAW-D/datml:nachricht/datml:datensegment/datml:satz[$satz-nummer]/
                datml:mmgr[@name = 'Autor']/datml:mm[@name = 'EMailAdresse']/datml:wert
                "/>

        <!-- Für die Erreichbarkeit des Autors entweder nur Satzebene als Datenquelle, oder nur Kopfebene: -->
        <xsl:choose>
            <!-- Wenn die Angaben auf Satzebene nicht vorhanden sind: Verwende Daten aus der Kopfebene -->
            <xsl:when test="empty(($telefon-nummer-satz, $email-satz))">
                <xsl:if test="$debug">
                    <xsl:comment>Für die Daten zur Erreichbarkeit des Autors wurde die Nachrichtenkopf-Ebene der Ursprungsnachricht als Datenquelle herangezogen.</xsl:comment>
                </xsl:if>

                <!-- E-Mail? -->
                <xsl:if test="/datml:DatML-RAW-D/datml:absender/datml:kontakt/datml:email">
                    <xga:erreichbarkeit>
                        <xga:kanal>
                            <code>01<!-- E-Mail --></code>
                        </xga:kanal>
                        <xga:kennung>
                            <xsl:value-of
                                select="/datml:DatML-RAW-D/datml:absender/datml:kontakt/datml:email"
                            />
                        </xga:kennung>
                    </xga:erreichbarkeit>
                </xsl:if>

                <!-- Telefon? -->
                <xsl:if test="/datml:DatML-RAW-D/datml:absender/datml:kontakt/datml:telefon">
                    <xga:erreichbarkeit>
                        <xga:kanal>
                            <code>02<!-- Telefon --></code>
                        </xga:kanal>
                        <xga:kennung>
                            <xsl:value-of
                                select="/datml:DatML-RAW-D/datml:absender/datml:kontakt/datml:telefon"
                            />
                        </xga:kennung>
                    </xga:erreichbarkeit>
                </xsl:if>
            </xsl:when>

            <!-- Mindestens entweder Telefon oder E-Mail (oder beide) sind auf Satzebene angegeben, also verwende diese. -->
            <xsl:otherwise>
                <xsl:if test="$debug">
                    <xsl:comment>Für die Daten zur Erreichbarkeit des Autors wurde die Satzebene der Ursprungsnachricht als Datenquelle herangezogen.</xsl:comment>
                </xsl:if>

                <!-- E-Mail? -->
                <xsl:if test="$email-satz">
                    <xga:errreichbarkeit>
                        <xga:kanal>
                            <code>01<!-- E-Mail --></code>
                        </xga:kanal>
                    </xga:errreichbarkeit>
                    <xga:kennung>
                        <xsl:value-of select="$email-satz"/>
                    </xga:kennung>
                </xsl:if>

                <!-- Telefon? -->
                <xsl:if test="$telefon-nummer-satz">
                    <xga:errreichbarkeit>
                        <xga:kanal>
                            <code>01<!-- Telefon --></code>
                        </xga:kanal>
                    </xga:errreichbarkeit>
                    <xga:kennung>
                        <xsl:value-of select="concat($telefon-vorwahl-satz, $telefon-nummer-satz)"/>
                    </xga:kennung>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- Der Behördenname des Autors auf Satzebene ist dort eine mandatorische Angabe, kann also in jedem Fall auf Satzebene bestimmt werden -->
    <xsl:template name="xga:behoerdenname-autor-satz">
        <xsl:param name="satz-nummer" as="xs:integer" required="yes"/>

        <xsl:for-each
            select="
                /datml:DatML-RAW-D/datml:nachricht/datml:datensegment/datml:satz[$satz-nummer]/
                datml:mmgr[@name = 'Autor']/datml:mm[@name = 'NameOrganisation']/datml:wert">
            <xga:behoerdenname>
                <xsl:value-of select="."/>
            </xga:behoerdenname>
        </xsl:for-each>
    </xsl:template>

    <!-- Registergericht -->
    <xsl:template match="datml:mm[@name = 'GerichtSchluessel']">
        <xga:gerichtSchluessel listVersionID="3">
            <code>
                <xsl:value-of select="./datml:wert"/>
            </code>
        </xga:gerichtSchluessel>
    </xsl:template>

    <!-- Geschlecht mit listVersionID (XGA-433) -->
    <xsl:template match="datml:mm[@name = 'Geschlecht']">
        <xga:geschlecht listVersionID="1">
            <code>
                <xsl:value-of select="./datml:wert"/>
            </code>
        </xga:geschlecht>
    </xsl:template>

    <!-- Hilfsfunktion, die für einen gegebenen Schlüssel (code) den zugehörigen Wert aus seiner Codeliste ausliest.
         Der Code wird gesucht in der im Codeliste-Dokument als Schlüssel referenzierten Spalte -->
    <xsl:function name="xga:codelisten-wert" as="xs:string?">
        <xsl:param name="urn" as="xs:string"/>
        <xsl:param name="schluessel" as="xs:string"/>
        <xsl:param name="abzufragende-spalte" as="xs:string"/>

        <xsl:variable name="betrachtete-codeliste" as="element(gc:CodeList)"
            select="$codelisten[Identification/CanonicalUri = $urn]"/>

        <xsl:variable name="schluessel-spalte" as="xs:string"
            select="$betrachtete-codeliste/ColumnSet/Key[1]/ColumnRef/@Ref"/>

        <xsl:variable name="gesuchte-zeile"
            select="$betrachtete-codeliste/SimpleCodeList/Row[Value[@ColumnRef = $schluessel-spalte] and Value[SimpleValue = $schluessel]]"
            as="element(Row)?"/>

        <xsl:sequence
            select="($gesuchte-zeile/Value[@ColumnRef = $abzufragende-spalte]/SimpleValue, '(KEIN CODELISTEN-LITERAL ZUGEORDNET!)')[1]"
        />
    </xsl:function>

    <!-- Hilfsfunktion, die die listVersionID einer im Verzeichnis codelisten abgelegten Codeliste der angegebenen URN zurückgibt.
         Der Defaultwert wird zurückgegeben, wenn $code-name-hinzufuegen = false() in den Parametern am Kopf dieses Skriptes gesetzt wurde. -->
    <xsl:function name="xga:listVersionID" as="xs:string?">
        <xsl:param name="urn" as="xs:string"/>

        <xsl:sequence
            select="($codelisten[Identification/CanonicalUri = $urn]/Identification/Version, '2018-01-31')[1]"
        />
    </xsl:function>

    <!-- Hilfsfunktion, die eine 12-stellige MELDID (Regionalschlüssel) in einen 8-stelligen AGS umwandelt, indem die Stellen 1-5 und 10-12 zusammengefügt werden  -->
    <xsl:function name="xga:meldid12-zu-ags8" as="xs:string">
        <xsl:param name="meldid" as="xs:string"/>

        <xsl:sequence select="concat(substring($meldid, 1, 5), substring($meldid, 10, 3))"/>

    </xsl:function>
</xsl:stylesheet>
